#!/usr/bin/perl

#Purpose: Clean out results directory for FoldMiner/LOCK 2. Directories will be deleted
#a certain number of hours after the LOCK or FoldMiner search finishes. 
#Ideally, we'd generate a warning 24 hours before deletion by modifying results.html

#Algorithm: The FoldMiner server places a file called "done" in each results directory after 
#the search finishes. If that file is more than 72 hours old, the directory is deleted. 

#Caveat: Directories 0000 and 1000 are never deleted. These are example runs.

#file age threshold for LOCK runs, in hours:
$lock_idle_threshold = 72; #in hours


#file age threshold for FoldMiner runs, in hours:
$foldminer_idle_threshold = 72; #in hours


#subdirectories in this directory will be cleaned up (if older than specified
#time)
$directory_to_clean = "/var/www/html/FoldMiner2/results";

#some variables
$rm = "/bin/rm";

&main;

sub main {
  my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$idst);

  #get the current system time
  ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
	localtime(time);
  
  deleteProtocol();
}

sub deleteProtocol {
  my($file,$dir, @resultsdirs, @files,  $threshold);

  #get a list of subdirectories in the results directory;
  opendir(DIR, $directory_to_clean);
  @resultsdirs = readdir(DIR);
  close(DIR);
  
  foreach $dir (@resultsdirs) {
    #if directory name is not a number, this is not a results directory
    next if($dir !~ /^[0-9]+/);

    #don't clean out directories containing examples
    next if($dir =~/^[10]000$/);

    if(-e("$directory_to_clean/$dir/options.list")) { #if  foldminer run
      deleteFoldMinerRun($dir);
    }else {
      deleteLockRun($dir);
    }
  }
}

#can't tell if user has been looking at results unless we place a file in there giving access time
sub deleteLockRun {
  my($dir) = @_;
  my($file_age);

  $file = "$directory_to_clean/$dir/done";
  if(! -e $file) {
    return; #search not finished
  }

  $current_time = time();
  @results = (stat $file);
  $access_time = $results[8];
  $file_idle_time = $current_time-$access_time; #in seconds
  $file_idle_time /= 360.0; #in hours
  if($file_idle_time > $lock_idle_threshold) {
    `$rm -rf $directory_to_clean/$dir`;
  }


}


sub deleteFoldMinerRun {
  my($dir) = @_;
  my(@results, $access_time, $current_time);
  my($file_idle_time);
  my($file);

  $file =  "$directory_to_clean/$dir/done";
  if(! -e $file) {
    return; #search not finished
  }

  #look at motif files and get the latest one? For now, same as lock delete
  $current_time = time();
  @results = (stat $file);
  $access_time = $results[8];
  $file_idle_time = $current_time-$access_time; #in seconds
  $file_idle_time /= 3600.0; #in hours
  if($file_idle_time > $foldminer_idle_threshold) {
    `$rm -rf $directory_to_clean/$dir`;
 }
  
}
