#!/usr/bin/perl

# Copyright 2003 The Board of Trustees of
# The Leland Stanford Junior University. All Rights Reserved.

# Communication to Jessica Shapiro (jessicas@stanford.edu)


#system commands used by this script:
#you may need to switch to full path names for some shells
$rm = "/bin/rm";

$mainfile = shift(@ARGV);
($outputfile) = $mainfile =~ /^(.*)\.out/;
$outputfile = "$outputfile.pdb";

open(MAINFILE, "<$mainfile");
open(OUT, ">$outputfile");
    
while ($line = <MAINFILE>) {
  
  chop($line);
  @fields = split /\s+/, $line;
  
  if ($fields[0] eq "Query:") {
    $queryfile = $fields[1];
    if ($fields[2] eq "Chain:") {
      $querychain = $fields[3];
    }
  }
  
  if ($fields[0] eq "Target:") {
    $targetfile = $fields[1];
    if ($fields[2] eq "Chain:") {
      $targetchain = $fields[3];
    }
  }
	
  if (($fields[0] eq "Transformation") && ($fields[3] eq "query")) {
    
    for ($i = 0; $i < 4; $i++) {
      $newline = <MAINFILE>;
      chop($newline);
      @newfields = split /\s+/, $newline;
      
      $qmat[$i][0] = $newfields[0];
      $qmat[$i][1] = $newfields[1];
      $qmat[$i][2] = $newfields[2];
    }
  }

  if (($fields[0] eq "Transformation") && ($fields[3] eq "target")) {
    
    for ($i = 0; $i < 4; $i++) {
      $newline = <MAINFILE>;
      chop($newline);
      @newfields = split /\s+/, $newline;
      
      $tmat[$i][0] = $newfields[0];
      $tmat[$i][1] = $newfields[1];
      $tmat[$i][2] = $newfields[2];
    }
  }

  if(($fields[0] eq "Query") && ($fields[1] eq "Replacement")) {
      $newline = <MAINFILE>;
      $index = 0;
      while($newline !~/Target/) {
	  ($number,$x,$y,$z) = split(/\s+/,$newline);
#	  print "$number\n";
	  $query_replace_xcoords[$number] = $x;
	  $query_replace_ycoords[$number] = $y;
	  $query_replace_zcoords[$number] = $z;
	  $query_replacements[$index] = $number;
	  $index++;
	  $newline = <MAINFILE>;
	  chomp($newline);
      }
      $index = 0;
      while($newline = <MAINFILE>) {
	  chomp($newline);
	  ($number,$x,$y,$z) = split(/\s+/,$newline);
	  $target_replace_xcoords[$number] = $x;
	  $target_replace_ycoords[$number] = $y;
	  $target_replace_zcoords[$number] = $z;
	  $target_replacements[$index] = $number;
	  $index++;
      }
  }
}
  print "$qmat[3][0] $qmat[3][1] $qmat[3][2]\n";
#print "query replacements: @query_replacements\n";
#print "target replacements: @target_replacements\n";

close(MAINFILE);

if (-e $queryfile) {
  open(Qfile, "<$queryfile");
}
else {
  if($queryfile =~ /^(.*)-(.)$/) {
    $shortqueryfile = $1;
    $querychain = $2;
  }

  `../FoldMiner2 $queryfile $$.temp1`;
  if (-e "$$.temp1") {
    open(Qfile, "<$$.temp1");
  }
  else {
    print "Sorry.  Could not find $queryfile\n";
    exit(0);
  }
  
  if (not defined $querychain) {
    ($querychain) = $queryfile =~ /^[1-9]...\-(.)/;
  }
}    
if (not defined $querychain) {
  $querychain = " ";
}


if (-e $targetfile) {
  open(Tfile, "<$targetfile");
}
else {
  `./findfile.pl $targetfile $$.temp2`;
  if (-e "$$.temp2") {
    open(Tfile, "<$$.temp2");
  }
  else {
    print "Sorry.  Could not find $targetfile\n";
    if (-e "$$.temp1") {
      `$rm $$.temp1`;
    }
    exit(0);
  }
  
  if (not defined $targetchain) {
    ($targetchain) = $targetfile =~ /^[1-9]...\-(.)/;
  }
}
if (not defined $targetchain) {
  $targetchain = " ";
}

print OUT "REMARK       $queryfile is Chain ID A  \n";
print OUT "REMARK       $targetfile is Chain ID B  \n";

#print "about to print atom info\n";
printATOMInfo(0);
printATOMInfo(1);
    
print OUT "END    \n";

close(OUT);
close(Qfile);
close(Tfile);

if (-e "$$.temp1") {
  `$rm $$.temp1`;
}
if (-e "$$.temp2") {
  `$rm $$.temp2`;
}

exit(0);

######################################################################


sub printATOMInfo() {
  
  if ($_[0] == 0) {
    $mychain = $querychain;	
    $myPtr = Qfile;
    $myoutchain = "A";
  }
  else {
    $mychain = $targetchain;
    $myPtr = Tfile;	
    $myoutchain = "B";
  }	
  
  seek $myPtr, 0, 0;
  
  while ($line = <$myPtr>) {
      
      $record = substr($line,0,6);
      
      if ($record eq "ATOM  " || $record eq "HETATM") {
	  $linechain = substr($line,21,1);
	  if (($mychain eq " ") || ($linechain eq $mychain)) {
	      substr($line,21,1) = $myoutchain;
	      $number = substr($line,22,4);
	      if((!(&replace_coords($number,$myoutchain)))) {
		  $myvec[0] = substr($line, 30, 8);
		  $myvec[1] = substr($line, 38, 8);
		  $myvec[2] = substr($line, 46, 8);

		  if (($myvec[0] ne "") && ($myvec[1] ne "") && ($myvec[2] ne "")) {
		      if ($_[0] == 0) {
			  applyMatToVect(\@qmat, \@myvec);
		      }
		      else {
			  applyMatToVect(\@tmat, \@myvec);
			}
		      
		      $tempstr = sprintf "%8.3f", $myvec[0];
		      substr($line, 30, 8) = $tempstr;
		      $tempstr = sprintf "%8.3f", $myvec[1];
		      substr($line, 38, 8) = $tempstr;
		      $tempstr = sprintf "%8.3f", $myvec[2];
		      substr($line, 46, 8) = $tempstr;
		      print OUT $line;
		  }		
	      }else {
		  if($myoutchain eq "A") {
		      $tempstr = sprintf("%8.3f",$query_replace_xcoords[$number]);
		      substr($line,30,8) = $tempstr;
		      $tempstr = sprintf("%8.3f",$query_replace_ycoords[$number]);
		      substr($line,38,8) = $tempstr;
		      $tempstr = sprintf("%8.3f",$query_replace_zcoords[$number]);
		      substr($line,46,8) = $tempstr;
		  }else {
		      $tempstr = sprintf("%8.3f",$target_replace_xcoords[$number]);
		      substr($line,30,8) = $tempstr;
		      $tempstr = sprintf("%8.3f",$target_replace_ycoords[$number]);
		      substr($line,38,8) = $tempstr;
		      $tempstr = sprintf("%8.3f",$target_replace_zcoords[$number]);
		      substr($line,46,8) = $tempstr;
		  }
		  print OUT $line;
	      }
	  }
      }
  }
}
  
sub replace_coords {
    my($number,$chain) = @_;
    my($i);
    my($replace);
    $replace = 0;
    $number+=0;
    if($chain eq "A") {
	for($i=0;$i<=$#query_replacements;$i++) {
	    if($query_replacements[$i] == $number) {
		$replace = 1;
		last;
	    }
	}
    }else {
	for($i=0;$i<=$#query_replacements;$i++) {
	    if($target_replacements[$i] == $number) {
		$replace = 1;
		last;
	    }
	}
    }
    return($replace);
}

sub applyMatToVect() {

    $t[0] = $_[1][0] * $_[0][0][0] + $_[1][1] * $_[0][1][0] + $_[1][2] * $_[0][2][0] + $_[0][3][0];
    $t[1] = $_[1][0] * $_[0][0][1] + $_[1][1] * $_[0][1][1] + $_[1][2] * $_[0][2][1] + $_[0][3][1];
    $t[2] = $_[1][0] * $_[0][0][2] + $_[1][1] * $_[0][1][2] + $_[1][2] * $_[0][2][2] + $_[0][3][2];

    $temp = $_[1][0]*$_[0][0][0];

    $_[1][0] = $t[0];
    $_[1][1] = $t[1];
    $_[1][2] = $t[2];

}
